import { h, createContext } from "preact";
import { useContext } from "preact/hooks";
const isServer = typeof window === "undefined";
export const HydrateContext = createContext(false);
export function withHydrate(Component, hydrationProps = {}) {
    const name = Component.displayName || Component.name;
    const { method } = hydrationProps;
    const Wrapped = (props, ref) => {
        const hydrateParent = useContext(HydrateContext);
        if (hydrateParent)
            throw new Error(`withHydrate() should only be called at the top-level of a Component tree. <${name} /> should not be nested within <${hydrateParent} />`);
        if (props.children && !["string", "number"].includes(typeof props.children))
            throw new Error(`withHydrate() is unable to serialize complex \`children\`. Please inline these children into <${name} />.`);
        const p = isServer ? `p=${JSON.stringify(props)}` : '';
        const m = isServer && method ? `m=${method}` : '';
        const Marker = 'hydrate-marker';
        return (h(HydrateContext.Provider, { value: name },
            isServer && (h(Marker, { dangerouslySetInnerHTML: { __html: `?h c=${name} ?` } })),
            h(Component, Object.assign({}, Object.assign(Object.assign({}, props), { ref }))),
            isServer && (h(Marker, { dangerouslySetInnerHTML: { __html: `?h ${[p, m].filter(v => v).join(' ')} ?` } }))));
    };
    Object.defineProperty(Wrapped, "name", { value: name, configurable: true });
    return Wrapped;
}
